use askama_axum::Template;
use axum::{extract::State, routing};

use ::entity::{debug, debug::Entity as DebugEntity};
use axum_htmx::HxRequest;
use sea_orm::*;

use crate::AppState;

async fn get_debug_entries(db: &DatabaseConnection) -> Result<Vec<debug::Model>, DbErr> {
    DebugEntity::find().all(db).await
}

async fn add_random_debug_entry(State(AppState { db_connection }): State<AppState>) {
    let random_entry = debug::ActiveModel {
        title: Set("Random title".to_string()),
        text: Set("Random text".to_string()),
        ..Default::default()
    };
    random_entry.insert(&db_connection).await.unwrap();
}

#[derive(Template)]
#[template(path = "debug.html")]
struct GetDebugTemplate {
    hx_request: bool,
    db_ping_status: bool,
    debug_entries_count: usize,
}

async fn debug(
    HxRequest(hx_request): HxRequest,
    State(AppState { db_connection }): State<AppState>,
) -> GetDebugTemplate {
    let db_ping_status = db_connection.ping().await.is_ok();
    let debug_entries = get_debug_entries(&db_connection).await.unwrap();
    GetDebugTemplate {
        hx_request,
        db_ping_status,
        debug_entries_count: debug_entries.len(),
    }
}

pub fn get_routes() -> axum::Router<crate::AppState> {
    axum::Router::new()
        .route("/", routing::get(debug))
        .route("/add_random", routing::post(add_random_debug_entry))
}
